# -*- coding: utf-8 -*-
""" FHCoreLibs > ADMIN > Base.py by Frankhood
    @author: Frankhood Business Solutions
"""
from __future__ import absolute_import, unicode_literals

import copy
import os

from django.conf import settings
from django.conf.urls import patterns, url
from django.contrib import admin, messages
from django.contrib.admin.filters import SimpleListFilter
from django.contrib.admin.views.main import ChangeList
from django.core.exceptions import ObjectDoesNotExist, MultipleObjectsReturned
from django.http import HttpResponseRedirect, HttpResponse, Http404
from django.utils import timezone
from django.utils.datastructures import SortedDict
from django.utils.encoding import force_unicode
from django.utils.html import escape, escapejs
from django.utils.translation import ungettext, ugettext as _

from ..models.mixins import DRAFT, PUBLISHED

from fhcore.utils.csv_utils import APIFileNameMixin


class isPublished(SimpleListFilter):
    title = _('Published')
    parameter_name = 'is_published'

    def lookups(self, request, model_admin):
        """
        Returns a list of tuples. The first element in each
        tuple is the coded value for the option that will
        appear in the URL query. The second element is the
        human-readable name for the option that will appear
        in the right sidebar.
        """
        return (
            ('True', _('Yes')),
            ('False', _('No')),
        )

    def queryset(self, request, queryset):
        """
        Returns the filtered queryset based on the value
        provided in the query string and retrievable via
        `self.value()`.
        """
        if self.value() == 'True' or self.value() == 'False':
            ids = []
            for obj in queryset:
                if obj.is_actual:
                    ids.append(obj.id)
            if self.value() == 'True':
                return queryset.filter(id__in=ids)
            else:
                return queryset.exclude(id__in=ids)
        if self.value():
            messages.add_message(request, messages.WARNING, _("Impossible to filter with the argument '{0}' for the key 'Published'").format(self.value()))


class PublicableModelAdmin(admin.ModelAdmin):
    publication_field = 'status'
    date_hierarchy = 'publication_start'
    actions = ['make_published', 'make_unpublished']

    def __init__(self, *args, **kwargs):
        super(PublicableModelAdmin, self).__init__(*args, **kwargs)
        if self.get_actual_status not in self.list_display:
            self.list_display = list(self.list_display) + [self.get_actual_status]
        # if self.get_actual_status not in self.list_editable:
        #    self.list_editable = list(self.list_editable) + [self.publication_field]
        if isPublished not in self.list_filter:
            self.list_filter = list(self.list_filter) + [isPublished]

    def get_actual_status(self, obj):
        return obj.is_actual
    get_actual_status.short_description = _('is published')
    get_actual_status.boolean = True

    def make_published(self, request, queryset):
        rows_updated = queryset.update(status=PUBLISHED, publication_start=timezone.now(), publication_end=None)
        self.message_user(request, ungettext('%(count)d entries was published',
                                             '%(count)d entries where published',
                                             rows_updated) % {'count': rows_updated})
    make_published.short_description = _('Publish selected entries')

    def make_unpublished(self, request, queryset):
        rows_updated = queryset.update(status=DRAFT, publication_start=None)
        self.message_user(request, ungettext('%(count)d entries was unpublished',
                                             '%(count)d entries where unpublished',
                                             rows_updated) % {'count': rows_updated})
    make_unpublished.short_description = _('Unpublish selected entries')


class OrderableModelAdmin(admin.ModelAdmin):
    order_field = 'order'
    ordering = ('order',)

    class Media:
        js = (
            'bower_components/jquery/dist/jquery.min.js',
            'bower_components/jqueryui/jquery-ui.min.js',
            'fhcore/admin/js/orderable.js',
        )

        css = {
            'all': ('bower_components/jqueryui/themes/ui-lightness/jquery-ui.css', )
        }

    def __init__(self, *args, **kwargs):
        super(OrderableModelAdmin, self).__init__(*args, **kwargs)
        if self.order_field not in self.list_display:
            self.list_display = list(self.list_display) + [self.order_field]
        if self.order_field not in self.list_editable:
            self.list_editable = list(self.list_editable) + [self.order_field]


class ConfigurableWidgetsMixinAdmin(object):

    dbfield_overrides = {}  # {'content':{'widget':TinyMCE()}}
    fkfield_overrides = {}  # {'content':{'widget':TinyMCE()}}

    def formfield_for_dbfield(self, db_field, **kwargs):
        if (self.dbfield_overrides and
                db_field.name in self.dbfield_overrides):
            kwargs.update(
                self.dbfield_overrides[db_field.name]
            )
        return super(ConfigurableWidgetsMixinAdmin, self).formfield_for_dbfield(
            db_field, **kwargs)

    def formfield_for_foreignkey(self, db_field, request=None, **kwargs):
        if (self.fkfield_overrides and
                db_field.name in self.fkfield_overrides):
            kwargs.update(
                self.fkfield_overrides[db_field.name]
            )
        return super(ConfigurableWidgetsMixinAdmin, self).formfield_for_foreignkey(
            db_field, request, **kwargs)


class LocaleModelAdmin(admin.ModelAdmin):
    locale_field = 'lang'

    def __init__(self, *args, **kwargs):
        super(LocaleModelAdmin, self).__init__(*args, **kwargs)
        if self.locale_field not in self.list_display:
            self.list_display = list(self.list_display) + [self.locale_field]
        if self.locale_field not in self.list_filter:
            self.list_filter = list(self.list_filter) + [self.locale_field]
        if self.locale_field not in self.search_fields:
            self.search_fields = list(self.search_fields) + [self.locale_field]

    def save_form(self, request, form, change):
        """
        Given a ModelForm return an unsaved instance. ``change`` is True if
        the object is being changed, and False if it's being added.
        """
        if request.method == 'POST' and "_saveasnew" in request.POST:
            if 'status' in form.cleaned_data and self.locale_field in form.changed_data:
                form.cleaned_data['status'] = DRAFT
        return super(LocaleModelAdmin, self).save_form(request, form, change)


class CountModelAdmin(admin.ModelAdmin):
    order_field = 'counter'
    ordering = ('-counter',)

    def __init__(self, *args, **kwargs):
        super(CountModelAdmin, self).__init__(*args, **kwargs)
        if self.order_field not in self.list_display:
            self.list_display = list(self.list_display) + [self.order_field]


class NoPreviewChangeList(ChangeList):
    preview_variable = 'is_preview'

    def get_query_set(self):
        return self.get_queryset()

    def get_queryset(self):
        qs = super(NoPreviewChangeList, self).get_query_set()
        if hasattr(self, 'preview_variable'):
            qs = qs.filter(**({getattr(self, 'preview_variable'): False}))
        return qs


class ConfigureRedirectMixinAdmin(object):

    def get_changelist(self, request, **kwargs):
        """
        Returns the ChangeList class for use on the changelist page.
        """
        return NoPreviewChangeList

    def redirect_for_continue(self, request, obj, post_url, msg=None, add=False):
        if msg:
            self.message_user(request, msg + ' ' + _("You may edit it again below."))
        return HttpResponseRedirect(post_url)

    def redirect_for_popup(self, request, obj, post_url=None, msg=None, add=False):
        pk_value = obj._get_pk_val()
        if post_url:
            return HttpResponseRedirect(post_url)
        else:
            return HttpResponse(
                '<script type="text/javascript">opener.dismissAddAnotherPopup(window, "%s", "%s");</script>' %
                # escape() calls force_unicode.
                (escape(pk_value), escapejs(obj))
            )

    def redirect_for_saveasnew(self, request, obj, post_url, msg=None, add=False):
        return HttpResponseRedirect(post_url)

    def redirect_for_saveanother(self, request, obj, post_url, msg=None, add=False):
        return HttpResponseRedirect(post_url)

    def redirect_for_addanother(self, request, obj, post_url="../add/", msg=None, add=False):
        return HttpResponseRedirect(post_url)

    def redirect_for_other(self, request, obj, msg, add):
        self.message_user(request, msg)

        # Figure out where to redirect. If the user has change permission,
        # redirect to the change-list page for this object. Otherwise,
        # redirect to the admin index.
        if self.has_change_permission(request, None):
            post_url = '../'
        else:
            post_url = '../../../'
        return HttpResponseRedirect(post_url)

    def response_add(self, request, obj, post_url_continue='../%s/'):
        """
        Determines the HttpResponse for the add_view stage.
        """
        opts = obj._meta
        pk_value = obj._get_pk_val()

        msg = _('The %(name)s "%(obj)s" was added successfully.') % {'name': force_unicode(opts.verbose_name), 'obj': force_unicode(obj)}
        # Here, we distinguish between different save types by checking for
        # the presence of keys in request.POST.
        if request.POST.has_key("_continue"):
            if request.POST.has_key("_popup"):
                post_url_continue += "?_popup=1"
            return self.redirect_for_continue(request=request, obj=obj, post_url=post_url_continue % pk_value, msg=msg, add=True)
        if request.POST.has_key("_popup"):
            return self.redirect_for_popup(request=request, obj=obj, msg=msg, add=True)
        elif request.POST.has_key("_addanother"):
            self.message_user(request, msg + ' ' + (_("You may add another %s below.") % force_unicode(opts.verbose_name)))
            return self.redirect_for_saveanother(request=request, obj=obj, post_url=request.path, msg=msg, add=True)
        else:
            return self.redirect_for_other(request=request, obj=obj, msg=msg, add=True)

    def response_change(self, request, obj):
        """
        Determines the HttpResponse for the change_view stage.
        """
        opts = obj._meta
        pk_value = obj._get_pk_val()

        msg = _('The %(name)s "%(obj)s" was changed successfully.') % {'name': force_unicode(opts.verbose_name), 'obj': force_unicode(obj)}
        if request.POST.has_key("_continue"):
            if request.REQUEST.has_key('_popup'):
                return self.redirect_for_continue(request=request, obj=obj, post_url=request.path + "?_popup=1", msg=msg, add=False)
            else:
                return self.redirect_for_continue(request=request, obj=obj, post_url=request.path, msg=msg, add=False)
        elif request.POST.has_key("_saveasnew"):
            msg = _('The %(name)s "%(obj)s" was added successfully. You may edit it again below.') % {'name': force_unicode(opts.verbose_name), 'obj': obj}
            self.message_user(request, msg)
            return self.redirect_for_saveasnew(request=request, obj=obj, post_url="../%s/" % pk_value, msg=msg, add=False)
        elif request.POST.has_key("_addanother"):
            self.message_user(request, msg + ' ' + (_("You may add another %s below.") % force_unicode(opts.verbose_name)))
            return self.redirect_for_addanother(request=request, obj=obj, msg=msg, add=False)
        else:
            return self.redirect_for_other(request=request, obj=obj, msg=msg, add=False)


class ConfigurePreviewMixinAdmin(ConfigureRedirectMixinAdmin):

    save_add = False
    save_as = False
    preview_variable = 'is_preview'
    preview_view = None  # ogni admin che lo eredita deve inserire la vista di preview da richiamare dal js
    preview_reverse_name = None  # ogni admin che lo eredita deve inserire il reverse url per scopi futuri
    inline_fk_name = "related_entries"  # None
    inline_fk_reverse = "photo_gallery"  # None

    class Media:
        js = (
            'bower_components/jquery/dist/jquery.min.js',
            'fhcore/admin/js/is_preview.js'
        )

    def redirect_for_continue(self, request, obj, post_url, msg=None, add=False):
        if msg:
            self.message_user(request, msg + ' ' + _("You may edit it again below."))
        new_url = '/'.join(post_url.split('/')[:-2]) + '/%s/' % obj.id
        return HttpResponseRedirect(new_url)

    def get_urls(self):
        urls = super(ConfigurePreviewMixinAdmin, self).get_urls()
        my_urls = patterns('',
                           url(r'^preview/(?P<pk>\d+)/$', self.admin_site.admin_view(self.preview_view.as_view(has_preview_permission=True)), name=self.preview_reverse_name)
                           )
        return my_urls + urls

    def queryset(self, request):
        qs = super(ConfigurePreviewMixinAdmin, self).queryset(request)
        return qs

    def get_old_obj_kw(self, obj, **kwargs):
        _bool = kwargs.get(self.preview_variable, None)
        tuples = self.model._meta.unique_together
        filter_kw = {}
        for tupla in tuples:
            if (_bool is not None) and self.preview_variable in tupla:
                filter_kw.update({self.preview_variable: _bool})
            for item in tupla:
                if not self.preview_variable == item:
                    filter_kw.update({item: getattr(obj, item)})
            break
        return filter_kw

    def save_model(self, request, obj, form, change):
        self.preview_enter = False
        self.old_obj = None
        self.curr_form = form
        try:
            kw = {self.preview_variable: False}
            old_obj = self.model._default_manager.get(**(self.get_old_obj_kw(obj, **kw)))
            # old_obj = self.model.objects.get(is_preview=False, slug = obj.slug, lang = obj.lang)
        except ObjectDoesNotExist:
            old_obj = None
        except MultipleObjectsReturned:
            raise Http404(u'You have to specify unique together in your model with preview variable into it')
        try:
            kw = {self.preview_variable: True}
            old_preview_obj = self.model._default_manager.get(**(self.get_old_obj_kw(obj, **kw)))
        except ObjectDoesNotExist:
            old_preview_obj = None
        except MultipleObjectsReturned:
            raise Http404(u'You have to specify unique together in your model with preview variable into it')
        if '_is_preview' in request.POST:
            if old_obj:
                if getattr(obj, self.preview_variable) == False:
                    # Sto attivando la preview
                    obj.id = None
                    self.preview_enter = True
            if old_preview_obj and (old_preview_obj.id != obj.id):
                old_preview_obj.delete()
            setattr(obj, self.preview_variable, True)
        else:
            if getattr(obj, self.preview_variable) == True:
                if old_obj:
                    self.old_obj = old_obj
            # setattr(obj,self.preview_variable,False)
        obj.save()
        # super(ConfigurePreviewMixinAdmin,self).save_model(request, obj, form, change)

    def save_formset(self, request, form, formset, change):
        """
        Given an inline formset save it to the database.
        """
        obj = form.instance
        if '_is_preview' in request.POST or getattr(obj, self.preview_variable):
            frms = formset.save(commit=False)
            for f in formset.forms:
                if f and f.instance and getattr(f.instance, self.inline_fk_name, None):
                    inst = f.instance
                    setattr(inst, self.inline_fk_name, obj)
                    if self.preview_enter:
                        inst.id = None
                    inst.save()
            for inst in frms:
                setattr(inst, self.inline_fk_name, obj)
                inst.save()
        elif self.old_obj:
            # Merge tra il vecchio oggetto e il nuovo
            frms = formset.save(commit=False)
            for old_inline in getattr(self.old_obj, self.inline_fk_reverse).all():
                old_inline.delete()
            for f in formset.forms:
                if f and f.instance and getattr(f.instance, self.inline_fk_name, None):
                    inst = f.instance
                    setattr(inst, self.inline_fk_name, self.old_obj)
                    inst.save()
            for inst in frms:
                setattr(inst, self.inline_fk_name, self.old_obj)
                inst.save()
        else:
            return formset.save()

    def log_addition(self, request, object):
        super(ConfigurePreviewMixinAdmin, self).log_addition(request, object)
        # TODO: spostare la cancellazione alla fine dell'aggiornamento di tutti i formsets!
        if self.old_obj:
            object.delete()  # ho cancellato la preview
            object.id = self.old_obj.id  # sostituisco l'originale con la preview
            setattr(object, self.preview_variable, False)
            object.save()
            self.curr_form.save_m2m()

    def log_change(self, request, object, message):
        super(ConfigurePreviewMixinAdmin, self).log_change(request, object, message)
        # TODO: spostare la cancellazione alla fine dell'aggiornamento di tutti i formsets!
        if self.old_obj:
            object.delete()  # ho cancellato la preview
            object.id = self.old_obj.id  # sostituisco l'originale con la preview
            setattr(object, self.preview_variable, False)
            object.save()
            self.curr_form.save_m2m()

    # Operazione da Mixin
    def redirect_for_other(self, request, obj, msg, add):
        # opts = obj._meta
        if '_is_preview' in request.POST:
            # self.message_user(request, msg + ' ' + (_("You may add another %s below with same customer.") % force_unicode(opts.verbose_name)))
            # potresti fare il redirect alla preview, ma devi pensare ad un modo per tornare indietro
            return HttpResponseRedirect("../%(id)s/?%(preview_var)s=%(id)s" % {'id': obj.id, 'preview_var': self.preview_variable})
        else:
            return super(ConfigurePreviewMixinAdmin, self).redirect_for_other(request, obj, msg, add)


class EmptyValueMixinAdmin(object):

    empty_values = {}

    def formfield_for_foreignkey(self, db_field, request=None, **kwargs):
        if hasattr(self, 'empty_values') and db_field.name in self.empty_values.keys():  # TODO: possibile non ci sia un modo + semplice per cambiare la empty_label?
            kwargs['empty_label'] = self.empty_values[db_field.name]
        return super(EmptyValueMixinAdmin, self).formfield_for_foreignkey(db_field, request, **kwargs)


class AfterSaveMixinAdmin(object):

    def log_addition(self, request, object):
        super(AfterSaveMixinAdmin, self).log_addition(request, object)
        self.after_save(request, object, add=True)

    def log_change(self, request, object, message):
        super(AfterSaveMixinAdmin, self).log_change(request, object, message)
        self.after_save(request, object, add=False)

    def after_save(self, request, object, add):
        # Permette di gestire le operazioni da eseguire dopo aver salvato un oggetto e tutti suoi inline
        pass


try:
    from rest_framework import generics, serializers
    from rest_framework_csv import *
    from .renderers import FHCSVRenderer
    
    class CSVExporterModelAdminMixin(object):
        """
        Il seguente admin mixin permetterà di aggiungere nelle action quella di export CSV
        il serializzatore potrà essere passato o creato di default
        la vista sarà creata inline e nn avrà bisogno di essere passata
        bisogna implementare la possibilità di specificare solo:
            - nome file *.csv
            - queryset (passata dalla change_list)
            - campi da stampare (che di default saranno quelli del list_display)
            - # tutti i campi da stampare se settings.DEBUG (che di default saranno quelli del list_display)
        """
        actions = ['export_data_in_csv', ]
        csvexporter_view = None
        csv_filename = None
        csv_delimiter = b';'
        csv_dialect = 'excel'
        csv_quotechar = b'"'

        def get_csv_filename(self):
            return self.csv_filename or "{0}_export.csv".format(self.model.__name__.lower())

        def get_csv_list_display(self):
            return tuple(self.list_display)

        def get_csv_list_fields(self):
            return [x if isinstance(x, basestring) else x[0] for x in self.get_csv_list_display()]

        def get_csvexporter_view(self):
            csvexporter_view = self.csvexporter_view
            if csvexporter_view is not None:
                return csvexporter_view

            assert self.model is not None, \
                "'%s' should either include a 'csvexporter_view' attribute, " \
                "or use the 'model' attribute as a shortcut for " \
                "automatically generating a csv exporter class." \
                % self.__class__.__name__

            class _AdminCSVSerializer(serializers.ModelSerializer):
                model_admin = self

                class Meta:
                    model = self.model
                    fields = self.get_csv_list_fields()

                def __init__(self, *args, **kwargs):
                    self._model_admin_functions = []
                    self._model_functions = []
                    return super(_AdminCSVSerializer, self).__init__(*args, **kwargs)

                def get_fields(self):
                    """
                    Returns the complete set of fields for the object as a dict.

                    This will be the set of any explicitly declared fields,
                    plus the set of fields returned by get_default_fields().
                    """
                    ret = SortedDict()

                    # Get the explicitly declared fields
                    base_fields = copy.deepcopy(self.base_fields)
                    for key, field in base_fields.items():
                        ret[key] = field

                    # Add in the default fields
                    default_fields = self.get_default_fields()
                    for key, val in default_fields.items():
                        if key not in ret:
                            ret[key] = val

                    # If 'fields' is specified, use those fields, in that order.
                    self._model_admin_functions = []
                    self._model_functions = []
                    if self.opts.fields:
                        assert isinstance(self.opts.fields, (list, tuple)), '`fields` must be a list or tuple'
                        new = SortedDict()
                        for key in self.opts.fields:
                            # new[key] = ret[key] # original code
                            ####### CUSTOM CODE #############
                            try:
                                new[key] = ret[key]
                            except KeyError as ex:
                                if hasattr(self.model_admin, key):
                                    self._model_admin_functions.append(key)
                                elif hasattr(self.opts.model, key):
                                    self._model_functions.append(key)
                                else:
                                    raise KeyError()
                            ####### endof CUSTOM CODE #############
                        ret = new
                    if self.opts.exclude:
                        assert isinstance(self.opts.exclude, (list, tuple)), '`exclude` must be a list or tuple'
                        for key in self.opts.exclude:
                            try:
                                ret.pop(key, None)
                            except KeyError as ex:
                                if hasattr(self.model_admin, key):
                                    self._model_admin_functions.remove(key)
                                elif hasattr(self.opts.model, key):
                                    self._model_functions.remove(key)

                    for key, field in ret.items():
                        field.initialize(parent=self, field_name=key)

                    return ret

                def to_native(self, obj):
                    ret = super(_AdminCSVSerializer, self).to_native(obj)
                    for x in self._model_admin_functions:
                        ret[x] = getattr(self.model_admin, x)(obj)
                    for x in self._model_functions:
                        if callable(getattr(obj, x)):
                            ret[x] = getattr(obj, x)()
                        else:
                            ret[x] = getattr(obj, x)
                    return ret

            class _AdminCSVRenderer(FHCSVRenderer):
                custom_headers = self.get_csv_list_display()
                csv_delimiter = self.csv_delimiter or ';'
                csv_dialect = self.csv_dialect or 'excel'
                csv_quotechar = self.csv_quotechar or '"'

            class DefaultCSVExporterView(APIFileNameMixin, generics.ListAPIView):
                model = self.model
                renderer_classes = (_AdminCSVRenderer,)
                serializer_class = _AdminCSVSerializer
                file_name = self.get_csv_filename()

                def get_file_name(self):
                    return self.file_name

                def post(self, *args, **kwargs):
                    return self.get(*args, **kwargs)
            # DefaultCSVExporterView.queryset=queryset
            return DefaultCSVExporterView

        def export_data_in_csv(self, request, queryset):
            csvexporter_view = self.get_csvexporter_view()
            request.method = 'GET'
            return csvexporter_view.as_view(queryset=queryset)(request=request)
        export_data_in_csv.short_description = 'Esporta dati in formato CSV'
except ImportError as ex:

    print('CSVExporterModelAdminMixin requires djangorestframework and djangorestframework_csv ')


    class CSVExporterModelAdminMixin(object):
        pass
