# -*- coding: utf-8 -*-
""" Frankhood Business Solutions > fhcore > utils > thumbnail_utils.py by Frankhood
    @author: Frankhood Business Solutions
    @author: Giovanni Bottalico
    @author: Gaetano D'Onghia
"""
from __future__ import unicode_literals
import logging
import mimetypes
import os
import stat
import posixpath
import re
try:
    from urllib.parse import unquote
except ImportError:     # Python 2
    from urllib import unquote

from django.conf import settings
from django.http import (CompatibleStreamingHttpResponse, Http404,
    HttpResponse, HttpResponseRedirect, HttpResponseNotModified)
from django.template import loader, Template, Context, TemplateDoesNotExist
from django.utils.http import http_date, parse_http_date
from django.utils.translation import ugettext as _, ugettext_noop
from django.views.static import was_modified_since, directory_index

logger = logging.getLogger(__name__)


try:
    from easy_thumbnails.files import get_thumbnailer
    from easy_thumbnails.storage import ThumbnailFileSystemStorage
    
    PRIVATE_ROOT = getattr(settings, 'PRIVATE_ROOT', settings.MEDIA_ROOT)
    PRIVATE_URL  = getattr(settings, 'PRIVATE_URL', settings.MEDIA_URL)
    ThumbnailStorage = ThumbnailFileSystemStorage(location=PRIVATE_ROOT, 
                                                  base_url= PRIVATE_URL)
    
    def factory_easythumbnail(img_field_name, default_size=(50,50), **defaults_kwargs):
        """
        instead of creaating a lot of property for serialized object because of thumbnail dim,
        you can so call this factory passing the size and crop properties
        Usage: 
        in your models.py:
             my_image = models.ImageField(...) 
             image_thumbnail = factory_easythumbnail('my_image',(108,108)) # my image could be a property, too
        in the view (or serializer or whrever you want):
             image_thumbnail_medium_url = obj.image_thumbnail(dims=(300,200))
             image_thumbnail_small_url = obj.image_thumbnail(dims=(50,50),crop="smart")
        """
        def get_obj_thumbnail(self, **img_kwargs):#dims=(892,412), crop='smart'):
            img_opts = defaults_kwargs.copy() or {}
            img_opts.update(img_kwargs)
            img = getattr(self,img_field_name)
            dims = img_opts.get('dims',default_size)
            crop = img_opts.get('crop','smart')
            snapshot_url = ''
            if img:
                try:
                    thumb_opts = {'size':dims, 'crop':crop, 'upscale':'True'}
                    # crea il thumbnail dello snapshot
                    snapshot_thumbnailer = get_thumbnailer(img)
                    snapshot_thumbnailer.thumbnail_storage = ThumbnailStorage
                    snapshot_thumb = snapshot_thumbnailer.get_thumbnail(thumb_opts)
                    snapshot_url = snapshot_thumb.url
                except Exception as ex:
                    snapshot_url = "{0}".format(ex)
                    #pass
                    #snapshot_url = image_field.url
            return snapshot_url
        return get_obj_thumbnail
    
except ImportError, ex:
    pass

try:
    from sorl.thumbnail import get_thumbnail 
    PRIVATE_ROOT = getattr(settings, 'PRIVATE_ROOT', settings.MEDIA_ROOT)
    PRIVATE_URL  = getattr(settings, 'PRIVATE_URL', settings.MEDIA_URL)
    SorlThumbnailStorage = ThumbnailFileSystemStorage(location=PRIVATE_ROOT, 
                                                         base_url= PRIVATE_URL)
    
    def factory_sorlthumbnail(img_field_name, default_size="50x50", **defaults_kwargs):
        """
        instead of creaating a lot of property for serialized object because of thumbnail dim,
        you can so call this factory passing the size and crop properties
        Usage: 
        in your models.py:
             my_image = models.ImageField(...)
             image_thumbnail = factory_sorlthumbnail('my_image',"108x108") # my image could be a property, too
        in the view (or serializer or whrever you want):
             image_thumbnail_medium_url = obj.image_thumbnail(dims="300x200")
             image_thumbnail_small_url = obj.image_thumbnail(dims="50x50",crop="smart")
        """
        def get_obj_thumbnail(self, **img_kwargs):#dims=(892,412), crop='smart'):
            img_opts = defaults_kwargs.copy() or {}
            img_opts.update(img_kwargs)
            img = getattr(self,img_field_name)
            geometry = img_opts.get('dims',default_size)
            crop = img_opts.get('crop','smart')
            snapshot_url = ''
            if img:
                try:
                    thumb_opts = {'crop':crop}
                    # crea il thumbnail dello snapshot
                    snapshot_thumb = get_thumbnail(img, geometry, **thumb_opts)
                    snapshot_url = snapshot_thumb.url
                except Exception as ex:
                    snapshot_url = "{0}".format(ex)
                    #pass
                    #snapshot_url = image_field.url
            return snapshot_url
        return get_obj_thumbnail
    
except ImportError, ex:
    pass
   
    

def image_serve(request, path, document_root=None, show_indexes=False):
    """
    Reimplementation of django.views.static.serve function
    The difference with the django.views.static.serve function is that 
    it uses mimetype image/jpeg as default and not octet/stream...
    """
    path = posixpath.normpath(unquote(path))
    path = path.lstrip('/')
    newpath = ''
    for part in path.split('/'):
        if not part:
            # Strip empty path components.
            continue
        drive, part = os.path.splitdrive(part)
        head, part = os.path.split(part)
        if part in (os.curdir, os.pardir):
            # Strip '.' and '..' in path.
            continue
        newpath = os.path.join(newpath, part).replace('\\', '/')
    if newpath and path != newpath:
        return HttpResponseRedirect(newpath)
    fullpath = os.path.join(document_root, newpath)
    if os.path.isdir(fullpath):
        if show_indexes:
            return directory_index(newpath, fullpath)
        raise Http404(_("Directory indexes are not allowed here."))
    if not os.path.exists(fullpath):
        raise Http404(_('"%(path)s" does not exist') % {'path': fullpath})
    # Respect the If-Modified-Since header.
    statobj = os.stat(fullpath)
    mimetype, encoding = mimetypes.guess_type(fullpath)
    mimetype = mimetype or 'image/jpeg'
    if not was_modified_since(request.META.get('HTTP_IF_MODIFIED_SINCE'),
                              statobj.st_mtime, statobj.st_size):
        return HttpResponseNotModified()
    response = CompatibleStreamingHttpResponse(open(fullpath, 'rb'), content_type=mimetype)
    response["Last-Modified"] = http_date(statobj.st_mtime)
    if stat.S_ISREG(statobj.st_mode):
        response["Content-Length"] = statobj.st_size
    if encoding:
        response["Content-Encoding"] = encoding
    return response

