# -*- coding: utf-8 -*-
""" FHCore  Project >  fhcore.views.fileupload_views by Frankhood
    @author: Frankhood Business Solutions
    @author: Gaetano D'Onghia
"""
from __future__ import absolute_import, print_function, unicode_literals
import base64
import datetime
import logging
import os
import cStringIO
from PIL import Image

from django.conf import settings
from django.core.files.base import ContentFile
from django.core.files.storage import default_storage
from django.core.files.uploadedfile import SimpleUploadedFile
from django.utils.translation import ugettext as _
from django.utils.datastructures import MultiValueDictKeyError
from django.views.generic.detail import SingleObjectMixin
from django.views.generic.base import View, ContextMixin

from fhcore.exceptions import FileNotSend, MaximumFileSizeExceeding
from fhcore.utils.backend import generate_code
from fhcore.views.base import JSONStructuredResponseMixin

logger = logging.getLogger(__name__)

"""
############################
#  BASE UPLOAD VIEWS #######
############################
"""
class FileGenericUtilityMixin(object):
    """
    Fornisce delle utils generiche comuni a tutte le viste di Upload
    """

    max_file_size = 5 * 2 ** 20  # in KB == 5 MB
    max_file_size_formatted = '5 MB'
    default_attachment_extension= 'jpg'
    
    def get_max_file_size(self):
        return self.max_file_size

    def validate_file(self, uploaded_file):
        if uploaded_file.size > self.get_max_file_size():
            raise MaximumFileSizeExceeding("File too big")

    def get_no_collition_filename(self, full_path):
        if not os.path.exists(full_path):
            return full_path
        dir, fname = os.path.split(full_path)
        file, ext = os.path.splitext(fname)
        i = 1
        while True:
            new_file_name = os.path.join(dir, "%s_%s%s" % (file, i, ext))
            if not os.path.exists(new_file_name):
                return new_file_name
            i += 1

    def get_custom_filename(self, **kwargs):
        return "image_uploaded_{0}.jpg".format(
            datetime.datetime.now().strftime("%d_%m_%Y__%H_%M"))

    def get_filename(self, uploaded_file):
        if hasattr(uploaded_file, 'name') and uploaded_file.name:
            return uploaded_file.name
        uploaded_file.name = self.get_custom_filename()
        return uploaded_file.name

    def get_attachment_extension(self, **kwargs):
        attachment = kwargs.get('attachment',None)
        if attachment and hasattr(attachment,'name') and attachment.name:
            ext = os.path.splitext(attachment.name)[1][1:]
            if ext:
                return ext
        return self.default_attachment_extension

class UploadFileAttachmentMixin(FileGenericUtilityMixin):
    """
    Mixin che prende dalla request un file uploadato in FILES o raw in POST
    """
    image_key = "files[]"  # settare questa come chiave, compatibile con jquery_file_upoload

    def create_file_from_raw_data(self, request, filename):
        raw_file = request.raw_post_data
        file, ext = os.path.splitext(filename)
        uploaded_file = SimpleUploadedFile(
            "%s%s" % (file, ext), raw_file, "image/%s" % ext[1:])
        return uploaded_file

    def get_uploaded_file(self, request, filename=None):
        try:
            uploaded_file = request.FILES[self.image_key]  # #files = request.FILES[u'files[]'] or request.FILES['file']
        except (MultiValueDictKeyError, KeyError), ex:
            uploaded_file = self.create_file_from_raw_data(request, filename)
        return uploaded_file

class UploadBase64AttachmentMixin(FileGenericUtilityMixin):
    """
    Da delle utility per iol salvataggio di file:
    x aver il file chiamare e funzioni in questa maniera:
    image = self.convert_base64_to_image(self.get_base64_image())
    """
    image_kwargs = "image"
    force_jpeg = True

    def convert_base64_to_image(self, base64image):
        image = base64image.encode('ascii').replace("\\n", "").replace("\n", "")
        if self.force_jpeg:
            cstring_image = cStringIO.StringIO()
            tmp = Image.open(ContentFile(base64.b64decode(str(image))))
            tmp.save(cstring_image, format='jpeg')
            converted_image = ContentFile(cstring_image.getvalue())
        else:
            converted_image = ContentFile(base64.b64decode(str(image)))  # urlsafe_b64decode(str(image)))
        return converted_image

    def get_base64_image(self):
        _post_data = self.request.POST.copy()
        try:
            base64_image = _post_data.get(self.image_kwargs)
            base64_image = base64_image.split(',')[1]
            return base64_image
        except KeyError as ex:
            logger.exception()
            raise FileNotSend()

    def get_uploaded_file(self, request):
        inmemory_file = self.convert_base64_to_image(self.get_base64_image())
        return inmemory_file

"""
########################################
##  DIRECTRLY TO STORAGE upload VIEWS ##
########################################
"""
class FileToStorageUtilityMixin(FileGenericUtilityMixin):
    upload_to = 'uploads'
    storage = default_storage
    override_permitted = False

    def get_storage(self):
        return self.storage

    def get_storage_location(self):
        """
        returns "{PROJECT_PATH}"
        """
        if self.get_storage():
            return self.get_storage().location
        else:
            return settings.PROJECT_PATH

    def get_storage_base_url(self):
        """
        returns "{MEDIA_URL}"
        """
        if self.get_storage():
            return self.get_storage().base_url
        else:
            return settings.MEDIA_URL

    def get_upload_to(self):
        """ /uploads/my_custom_directory/"""
        return self.upload_to

    def get_file_url(self, uploaded_file):
        """
        returns /{MEDIA_URL}/{UPLOAD_TO}/{FILE.NAME}
        """
        return os.path.join(
            self.get_storage_base_url(),
            self.get_upload_to(),
            self.get_filename(uploaded_file)
        )

    def get_file_fullpath(self, uploaded_file):
        """
        returns "/{PROJECT_PATH}/{UPLOAD_TO}/{FILE.NAME}"
        """
        return os.path.join(
            self.get_storage_location(),
            self.get_upload_to(),
            self.get_filename(uploaded_file)
        )

    def save_file(self, uploaded_file):
        temp_file_destination_fullpath = self.get_file_fullpath(uploaded_file)
        #logger.info("*****override_permitted******** : {0}".format(self.override_permitted))
        if not self.override_permitted:
            file_destination_fullpath = self.get_no_collition_filename(temp_file_destination_fullpath)
        else:
            file_destination_fullpath = temp_file_destination_fullpath
        #logger.info("++++++++self.get_storage()++++++++ : {0}".format(self.get_storage()))
        if self.get_storage():
            path = self.get_storage().save(file_destination_fullpath, uploaded_file)
        else:
            path = default_storage.save(file_destination_fullpath, uploaded_file)
        #logger.info("++++++++path++++++++ : {0}".format(path))
        return self.get_file_url(uploaded_file)


class UploadAttachmentBaseView(FileToStorageUtilityMixin,
                               ContextMixin, View):
    
    save_to_disk = True
    
    def post(self, request, *args, **kwargs):
        uploaded_file, filepath = (None, "")
        try:
            (uploaded_file, filepath) = self.process_data(request)
            if uploaded_file:
                self.check_success = True
            msg = "File Uploaded"
        except MaximumFileSizeExceeding as ex:
            logger.exception("MaximumFileSizeExceeding")
            msg = 'Dimensione del file eccessiva.. La dimensione massima consentita &egrave {0}'.format(self.max_file_size_formatted)
        except FileNotSend:
            logger.exception("FileNotSend")
            msg = 'File Non inviato'
        return self.render_to_response(
                    self.get_context_data(
                        **({'file': uploaded_file,
                            'filepath': filepath,
                            'message': msg})
                ))

    def process_data(self, request):
        uploaded_file = self.get_uploaded_file(request)
        if uploaded_file:
            self.validate_file(uploaded_file)
            if self.save_to_disk:
                filepath = self.save_file(uploaded_file)
                request.session['last_file_name'] = os.path.split(filepath)[1]
            else:
                filepath = None
            return (uploaded_file, filepath) or (None, None)
        raise FileNotSend("File Not Send")
                
class UploadFileAttachmentBaseView(UploadFileAttachmentMixin,
                                   UploadAttachmentBaseView):
    pass

class UploadBase64AttachmentBaseView(UploadBase64AttachmentMixin,
                                     UploadFileAttachmentBaseView):
    pass

class JSONStructuredResponseAttachmentToStorageMixin(JSONStructuredResponseMixin):

    content_type = ('text/html', 'application/json', )

    def add_success_metadata(self, **kwargs):
        return {'file': kwargs.get('file', ''),
                'filepath': kwargs.get('filepath', ''),
                'message': kwargs.get('message', '')}

    def add_failure_metadata(self, **kwargs):
        return {'message': kwargs.get('message', '')}
    
class UploadFileAttachmentJSONView(JSONStructuredResponseAttachmentToStorageMixin,
                                   UploadFileAttachmentBaseView):
    pass

class UploadBase64AttachmentJSONView(JSONStructuredResponseAttachmentToStorageMixin,
                                     UploadBase64AttachmentBaseView):
    pass

"""    
###########################################################################
#############  ATTACHMENT AND MODEL INTEGRATION  ##########################
###########################################################################
"""
class SingleObjectAttachmentMixin(SingleObjectMixin):

    file_field = ''
    model = None
    filename_prefix = 'video_'
    default_attachment_extension = 'jpg'
    force_custom_filename = False
    
    def get_object(self, queryset=None):
        if 'pk' in self.kwargs:  # UpdateView
            obj = super(SingleObjectAttachmentMixin, self).get_object(queryset)
            return obj
        else:  # CreateView
            return None
    
    def get_filename_prefix(self):
        return self.filename_prefix
    
    def get_attachment_extension(self, **kwargs):
        attachment = kwargs.get('attachment',None)
        if attachment and hasattr(attachment,'name') and attachment.name:
            ext = os.path.splitext(attachment.name)[1][1:]
            if ext:
                return ext
        return self.default_attachment_extension
    
    def get_attachment_filename(self, attachment=None, **kwargs):
        if not self.force_custom_filename and attachment and hasattr(attachment,'name') and attachment.name:
            return attachment.name
        if self.object and self.object.id:
            return "{0}{1}.{2}".format(self.get_filename_prefix(), self.object.id, self.get_attachment_extension(**{'attachment':attachment}))
        else:
            return "{0}{1}.{2}".format(self.get_filename_prefix(), generate_code(), self.get_attachment_extension(**{'attachment':attachment}))
    
    def get_object_creation_kwargs(self, **kwargs):
        return {}

    def create_object(self, **kwargs):
        edited_attachment = kwargs.get('uploaded_file',self.get_uploaded_file(self.request))
        new_obj = self.model(**self.get_object_creation_kwargs())
        new_obj.save()
        self.object = new_obj
        getattr(self.object, self.file_field).save(
                "{0}".format(self.get_attachment_filename(edited_attachment)), edited_attachment)
        return self.object

    def update_object(self):
        try:
            edited_file = self.get_edited_attachment()
            getattr(self.object, self.file_field).save(
                "{0}".format(self.get_attachment_filename(edited_file)), edited_file)
            return True
        except FileNotSend as ex:
            logger.exception()
            return False

    """
    def get_filename(self, uploaded_file):
        if self.object and self.object.id:
            if hasattr(self.object, self.file_field) and getattr(self.object, self.file_field, None):
                return
            return "image_{0}_{1}.jpg".format(self.object.id, datetime.datetime.now().strftime("%d_%m_%Y__%H_%M"))
        else:
            uploaded_file.name = self.get_custom_filename()
            return uploaded_file.name 
    """
    
class UploadFileBaseModelView(SingleObjectAttachmentMixin, View):

    def process_data(self, request):
        uploaded_file = self.get_uploaded_file(request)
        if uploaded_file:
            self.validate_file(uploaded_file)
            return uploaded_file
        raise FileNotSend("File Not Send")

    def update_business_logic(self):
        pass

    def post(self, request, *args, **kwargs):
        uploaded_file = None
        try:
            uploaded_file = self.process_data(request)
            logger.info("uploaded_file : %s"%uploaded_file)
            if uploaded_file:
                self.check_success = True
            msg = "File Uploaded"
        except MaximumFileSizeExceeding as ex:
            logger.exception("MaximumFileSizeExceeding")
            msg = 'Dimensione del file eccessiva.. La dimensione massima consentita &egrave {0}'.format(self.max_file_size_formatted)
        except FileNotSend:
            logger.exception("FileNotSend")
            msg = 'File Non inviato'
        self.object = self.get_object()
        if self.check_success:
            if self.object:
                self.update_object()
            else:
                self.object = self.create_object(**({'uploaded_file':uploaded_file}))
            self.update_business_logic()
            context = self.get_context_data(object=self.object,**{'message':msg})
        else:
            context = self.get_context_data(**{'message':msg})
        return self.render_to_response(context)
        """return self.render_to_response(
                    self.get_context_data(
                        **({'file': uploaded_file,
                            'filepath': filepath,
                            'message': msg})
                ))"""
    
        """def post(self, request, *args, **kwargs):
            self.object = self.get_object()
            if self.object:
                self.update_object()
            else:
                self.object = self.create_object(**self.get_object_creation_kwargs(**kwargs))
            context = self.get_context_data(object=self.object)
            return self.render_to_response(context)"""

class JSONStructuredResponseAttachmentModelMixin(JSONStructuredResponseMixin):
    content_type = ('application/json', 'text/html')

    def get_check_success(self, **kwargs):
        return self.object and getattr(self.object, self.file_field)

    def add_success_metadata(self, **kwargs):
        return {'filepath': self.object and
                            getattr(self.object, self.file_field) and
                            getattr(self.object, self.file_field).url or "",
                'message': kwargs.get('message', '')}

    def add_failure_metadata(self, **kwargs):
        return {'message': kwargs.get('message', '')}
  
class UploadFileAttachmentModelView(JSONStructuredResponseAttachmentModelMixin,
                                    UploadFileAttachmentMixin,
                                    UploadFileBaseModelView):
    """
    @TODO Tuning
    PAY ATTENTION
    THIS IS NOT TESTED
    USE fhcore.views.file_upload.UploadAttachmentModelView instead 
    """
    file_field = 'file'
    model = None
    
class UploadBase64AttachmentModelView(JSONStructuredResponseAttachmentModelMixin,
                                      UploadBase64AttachmentMixin,
                                      UploadFileBaseModelView):
    """
    @TODO Tuning
    PAY ATTENTION
    THIS IS NOT TESTED
    USE fhcore.views.file_upload.UploadBase64AttachmentModelView instead
    """
    file_field = 'image'
    model = None

"""
###########################################################################
#######################     THUMBNAIL MIXINS  #############################
###########################################################################
"""
try:
    from sorl.thumbnail.shortcuts import get_thumbnail

    class SorlThumbnailPreviewMixin(object):
        image_geometry = "50x50"
        image_quality = 90
        crop = 'center'

        def get_image_geometry(self):
            return self.image_geometry

        def get_image_opts(self):
            return {
                    'crop': self.crop,
                    'quality': self.image_quality
                   }

        def get_thumbnail(self, image_path):
            return get_thumbnail(image_path, self.get_image_geometry(), **(self.get_image_opts()))

except ImportError as ex:
    logger.info("Sorl Thumbnail is not installed: %s" % ex)

try:
    from easy_thumbnails.files import get_thumbnailer

    class EasyThumbnailPreviewMixin(object):
        image_geometry = (50, 50)
        # image_quality = 90
        crop = 'center'
        thumbnail_storage = None

        def get_image_geometry(self):
            return self.image_geometry

        def get_image_opts(self):
            return {'size':self.get_image_geometry(),
                     'crop':'smart',
                     # 'quality' : self.image_quality
                     }

        def get_thumbnail_storage(self):
            return self.thumbnail_storage

        def get_thumbnail(self, image_path):
            thumbnailer = get_thumbnailer(image_path)
            if self.get_thumbnail_storage():
                thumbnailer.source_storage = self.get_storage()
                thumbnailer.thumbnail_storage = self.get_thumbnail_storage()
            return thumbnailer.get_thumbnail(self.get_image_opts()).url  # name

except ImportError as ex:
    logger.info("Easy Thumbnail is not installed: %s" % ex)
